use std::sync::Arc;

use collab_persistence::kv::rocks_kv::RocksCollabDB;

use flowy_database2::DatabaseManager2;
use flowy_document2::manager::{DocumentManager as DocumentManager2, DocumentUser};
use flowy_error::FlowyError;
use flowy_user::services::UserSession;

pub struct Document2DepsResolver();
impl Document2DepsResolver {
  pub fn resolve(
    user_session: Arc<UserSession>,
    _database_manager: &Arc<DatabaseManager2>,
  ) -> Arc<DocumentManager2> {
    let user: Arc<dyn DocumentUser> = Arc::new(DocumentUserImpl(user_session));

    Arc::new(DocumentManager2::new(user.clone()))
  }
}

struct DocumentUserImpl(Arc<UserSession>);
impl DocumentUser for DocumentUserImpl {
  fn user_id(&self) -> Result<i64, FlowyError> {
    self
      .0
      .user_id()
      .map_err(|e| FlowyError::internal().context(e))
  }

  fn token(&self) -> Result<String, FlowyError> {
    self
      .0
      .token()
      .map_err(|e| FlowyError::internal().context(e))
  }

  fn kv_db(&self) -> Result<Arc<RocksCollabDB>, FlowyError> {
    self.0.get_kv_db()
  }
}
